==> ThbgmExtractor-1.6.3.tar.bz2: ThbgmExtractor-1.6.3/core/Smdn.Windows.Forms.ThbgmPlayer/Smdn.Windows.Forms.ThbgmPlayer/EffectEditor.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2009-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Smdn.Windows.Forms.ThbgmPlayer {
  public partial class EffectEditor : UserControl {
    /*
     * properties
     */
    public Effect Effect {
      get { return effect; }
      set
      {
        if (effect == value)
          return;

        if (effect != null)
          effect.PropertyChanged -= Effect_PropertyChanged;

        effect = value;

        if (effect != null)
          effect.PropertyChanged += Effect_PropertyChanged;

        OnEffectChanged();
      }
    }

    /*
     * construction/destruction
     */
    public EffectEditor()
    {
      InitializeComponent();

      fadingCurveTypeDropDownList.SelectedIndexChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;

        updatingEffectProperty = true;

        effect.FadingCurveType = (sender as FadingCurveTypeDropDownList).SelectedItem;

        UpdateFadingCurveCoefficientRange(effect.FadingCurveType);

        switch (effect.FadingCurveType) {
          case FadingCurveType.Exponential:
          case FadingCurveType.Logarithmic:
            numericUpDownFadingCurveCoefficient.Value = 10.0m;
            effect.FadingCurveCoefficient = 10.0;
            break;

          case FadingCurveType.Trigonometrical:
            numericUpDownFadingCurveCoefficient.Value = 1.0m;
            effect.FadingCurveCoefficient = 1.0;
            break;

          default:
            numericUpDownFadingCurveCoefficient.Value = 1.0m;
            effect.FadingCurveCoefficient = 1.0;
            break;
        }

        updatingEffectProperty = false;
      };

      numericUpDownFadingCurveCoefficient.DecimalPlaces = 2;
      numericUpDownFadingCurveCoefficient.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.FadingCurveCoefficient = (double)(sender as NumericUpDown).Value;
        updatingEffectProperty = false;
      };

      timeSpanUpDownFadingStart.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownFadingStart.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownFadingStart.Minimum = TimeSpan.Zero;
      timeSpanUpDownFadingStart.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownFadingStart.Increment = new TimeSpan(0, 0, 0, 0, 200);
      timeSpanUpDownFadingStart.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.FadingStart = (sender as TimeSpanUpDown).Value;
        updatingEffectProperty = false;
      };

      timeSpanUpDownFadingLength.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownFadingLength.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownFadingLength.Minimum = TimeSpan.Zero;
      timeSpanUpDownFadingLength.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownFadingLength.Increment = new TimeSpan(0, 0, 0, 0, 200);
      timeSpanUpDownFadingLength.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.FadingLength = (sender as TimeSpanUpDown).Value;
        updatingEffectProperty = false;
      };

      timeSpanUpDownTrailingSilenceLength.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownTrailingSilenceLength.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownTrailingSilenceLength.Minimum = TimeSpan.Zero;
      timeSpanUpDownTrailingSilenceLength.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownTrailingSilenceLength.Increment = new TimeSpan(0, 0, 0, 0, 200);
      timeSpanUpDownTrailingSilenceLength.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.TrailingSilenceLength = (sender as TimeSpanUpDown).Value;
        updatingEffectProperty = false;
      };
    }

    private void OnEffectChanged()
    {
      this.Enabled = (effect != null);

      if (effect != null)
        UpdateValues();
    }

    private void UpdateValues()
    {
      updatingEffectProperty = true;

      var fadingCurveCoefficient = (decimal)effect.FadingCurveCoefficient; // keep current value

      for (var i = 0; i < fadingCurveTypeDropDownList.Items.Count; i++) {
        if (fadingCurveTypeDropDownList.Items[i] == effect.FadingCurveType)
          fadingCurveTypeDropDownList.SelectedIndex = i;
      }

      UpdateFadingCurveCoefficientRange(effect.FadingCurveType);

      if (numericUpDownFadingCurveCoefficient.Maximum < fadingCurveCoefficient)
        numericUpDownFadingCurveCoefficient.Value = numericUpDownFadingCurveCoefficient.Maximum;
      else if (numericUpDownFadingCurveCoefficient.Minimum > fadingCurveCoefficient)
        numericUpDownFadingCurveCoefficient.Value = numericUpDownFadingCurveCoefficient.Minimum;
      else
        numericUpDownFadingCurveCoefficient.Value = fadingCurveCoefficient;

      timeSpanUpDownFadingStart.Value = effect.FadingStart;
      timeSpanUpDownFadingLength.Value = effect.FadingLength;
      timeSpanUpDownTrailingSilenceLength.Value = effect.TrailingSilenceLength;

      UpdateTotalLength();

      updatingEffectProperty = false;
    }

    private void UpdateFadingCurveCoefficientRange(FadingCurveType fadingCurveType)
    {
      numericUpDownFadingCurveCoefficient.Minimum   = Effect.FadingCurveCoefficientUIMinValue[fadingCurveType];
      numericUpDownFadingCurveCoefficient.Maximum   = Effect.FadingCurveCoefficientUIMaxValue[fadingCurveType];
      numericUpDownFadingCurveCoefficient.Increment = Effect.FadingCurveCoefficientUIIncrementValue[fadingCurveType];

      switch (fadingCurveType) {
        case FadingCurveType.Exponential:
        case FadingCurveType.Logarithmic:
        case FadingCurveType.Trigonometrical:
          numericUpDownFadingCurveCoefficient.Enabled = true;
          break;

        default:
          numericUpDownFadingCurveCoefficient.Enabled = false;
          break;
      }
    }

    private void UpdateTotalLength()
    {
      labelTotalLength.Text = string.Format("{0:D}:{1:D2}:{2:D2}.{3:D3}", (int)effect.TotalLength.TotalHours, effect.TotalLength.Minutes, effect.TotalLength.Seconds, effect.TotalLength.Milliseconds);
    }

    private void Effect_PropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      if (sender == effect) {
        if (updatingEffectProperty)
          UpdateTotalLength();
        else
          UpdateValues();
      }
    }

    private Effect effect = null;
    private bool updatingEffectProperty = false;
  }
}

==> ThbgmExtractor-1.6.4.tar.bz2: ThbgmExtractor-1.6.4/core/Smdn.Windows.Forms.ThbgmPlayer/Smdn.Windows.Forms.ThbgmPlayer/EffectEditor.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2009-2017 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Smdn.Windows.Forms.ThbgmPlayer {
  public partial class EffectEditor : UserControl {
    /*
     * properties
     */
    public Effect Effect {
      get { return effect; }
      set
      {
        if (effect == value)
          return;

        if (effect != null)
          effect.PropertyChanged -= Effect_PropertyChanged;

        effect = value;

        if (effect != null)
          effect.PropertyChanged += Effect_PropertyChanged;

        OnEffectChanged();
      }
    }

    /*
     * construction/destruction
     */
    public EffectEditor()
    {
      InitializeComponent();

      fadingCurveTypeDropDownList.SelectedIndexChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.SetFadingCurve((sender as FadingCurveTypeDropDownList).SelectedItem, true);
        numericUpDownFadingCurveCoefficient.Value = (decimal)effect.FadingCurveCoefficient;
        UpdateFadingCurveCoefficientRange(effect.FadingCurveType);
        updatingEffectProperty = false;
      };

      numericUpDownFadingCurveCoefficient.DecimalPlaces = 2;
      numericUpDownFadingCurveCoefficient.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.SetFadingCurve((double)(sender as NumericUpDown).Value, true);
        updatingEffectProperty = false;
      };

      timeSpanUpDownFadingStart.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownFadingStart.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownFadingStart.Minimum = TimeSpan.Zero;
      timeSpanUpDownFadingStart.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownFadingStart.Increment = new TimeSpan(0, 0, 0, 0, 200);
      timeSpanUpDownFadingStart.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.FadingStart = (sender as TimeSpanUpDown).Value;
        updatingEffectProperty = false;
      };

      timeSpanUpDownFadingLength.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownFadingLength.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownFadingLength.Minimum = TimeSpan.Zero;
      timeSpanUpDownFadingLength.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownFadingLength.Increment = new TimeSpan(0, 0, 0, 0, 200);
      timeSpanUpDownFadingLength.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.FadingLength = (sender as TimeSpanUpDown).Value;
        updatingEffectProperty = false;
      };

      timeSpanUpDownTrailingSilenceLength.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownTrailingSilenceLength.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownTrailingSilenceLength.Minimum = TimeSpan.Zero;
      timeSpanUpDownTrailingSilenceLength.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownTrailingSilenceLength.Increment = new TimeSpan(0, 0, 0, 0, 200);
      timeSpanUpDownTrailingSilenceLength.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.TrailingSilenceLength = (sender as TimeSpanUpDown).Value;
        updatingEffectProperty = false;
      };
    }

    private void OnEffectChanged()
    {
      this.Enabled = (effect != null);

      if (effect != null)
        UpdateValues();
    }

    private void UpdateValues()
    {
      updatingEffectProperty = true;

      var fadingCurveCoefficient = (decimal)effect.FadingCurveCoefficient; // keep current value

      for (var i = 0; i < fadingCurveTypeDropDownList.Items.Count; i++) {
        if (fadingCurveTypeDropDownList.Items[i] == effect.FadingCurveType)
          fadingCurveTypeDropDownList.SelectedIndex = i;
      }

      UpdateFadingCurveCoefficientRange(effect.FadingCurveType);

      if (numericUpDownFadingCurveCoefficient.Maximum < fadingCurveCoefficient)
        numericUpDownFadingCurveCoefficient.Value = numericUpDownFadingCurveCoefficient.Maximum;
      else if (numericUpDownFadingCurveCoefficient.Minimum > fadingCurveCoefficient)
        numericUpDownFadingCurveCoefficient.Value = numericUpDownFadingCurveCoefficient.Minimum;
      else
        numericUpDownFadingCurveCoefficient.Value = fadingCurveCoefficient;

      timeSpanUpDownFadingStart.Value = effect.FadingStart;
      timeSpanUpDownFadingLength.Value = effect.FadingLength;
      timeSpanUpDownTrailingSilenceLength.Value = effect.TrailingSilenceLength;

      UpdateTotalLength();

      updatingEffectProperty = false;
    }

    private void UpdateFadingCurveCoefficientRange(FadingCurveType fadingCurveType)
    {
      numericUpDownFadingCurveCoefficient.Minimum   = Effect.FadingCurveCoefficientUIMinValue[fadingCurveType];
      numericUpDownFadingCurveCoefficient.Maximum   = Effect.FadingCurveCoefficientUIMaxValue[fadingCurveType];
      numericUpDownFadingCurveCoefficient.Increment = Effect.FadingCurveCoefficientUIIncrementValue[fadingCurveType];

      switch (fadingCurveType) {
        case FadingCurveType.Exponential:
        case FadingCurveType.Logarithmic:
        case FadingCurveType.Trigonometrical:
          numericUpDownFadingCurveCoefficient.Enabled = true;
          break;

        default:
          numericUpDownFadingCurveCoefficient.Enabled = false;
          break;
      }
    }

    private void UpdateTotalLength()
    {
      labelTotalLength.Text = string.Format("{0:D}:{1:D2}:{2:D2}.{3:D3}", (int)effect.TotalLength.TotalHours, effect.TotalLength.Minutes, effect.TotalLength.Seconds, effect.TotalLength.Milliseconds);
    }

    private void Effect_PropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      if (sender == effect) {
        if (updatingEffectProperty)
          UpdateTotalLength();
        else
          UpdateValues();
      }
    }

    private Effect effect = null;
    private bool updatingEffectProperty = false;
  }
}
